function plotDensity(minDens, densInc, densIters, nInstances, dataName, nmfName, svdName, capName, saveNameL1, saveNameFro, addLegend)

% Params.
FontSize = 30;
mn = 10;
LW = 1.2;
MarkerSize = 10;
xLegendShift = 0.04;
yLegendShift = 0.04;
% axisShift = params.inc_ / 6;


% Find densities range.
densities = nan(densIters, 1);
for i = 1 : densIters
    densities(i) = minDens + (i-1)*densInc;
end

load(dataName);

load(capName);
load(nmfName);
load(svdName);

resultsCap = nan(densIters, nInstances);
resultsNMF = nan(densIters, nInstances);
resultsSVD = nan(densIters, nInstances);

count = 1;
names{count} = '$$\texttt{Capricorn}$$';
count = count + 1;

names{count} = '$$\texttt{NMF}$$';
count = count + 1;

names{count} = '$$\SVD$$';


%% L1 error
for i = 1 : densIters
    for j = 1 : nInstances
        resultsCap(i, j) = L1(Aorig{i}{j}, Bcap{i}{j}, Ccap{i}{j}, 0);
        resultsNMF(i, j) = sum(sum(abs(Aorig{i}{j} - Bnmf{i}{j}*Cnmf{i}{j}))) / sum(sum(abs(Aorig{i}{j})));
        resultsSVD(i, j) = sum(sum(abs(Aorig{i}{j} - Bsvd{i}{j}*Csvd{i}{j}))) / sum(sum(abs(Aorig{i}{j})));
    end
end
errorbar(densities, mean(resultsCap, 2), std(resultsCap, 0, 2), '-xr', ...
    'LineWidth', LW, 'markers', MarkerSize);
hold on
errorbar(densities, mean(resultsNMF, 2), std(resultsNMF, 0, 2), '-xb', ...
    'LineWidth', LW, 'markers', MarkerSize);
errorbar(densities, mean(resultsSVD, 2), std(resultsSVD, 0, 2), '-xk', ...
    'LineWidth', LW, 'markers', MarkerSize);
if addLegend == 1
    h = legend('Capricorn', 'NMF', 'SVD', 'fontsize', FontSize+30);
    set(h,'FontSize', FontSize - mn);
    h.Position(1) = h.Position(1) + xLegendShift;
    h.Position(2) = h.Position(2) + yLegendShift;
    legend boxoff
end

xlabel('Factor density', 'interpreter', 'latex', 'fontsize', FontSize);
ylabel('L1 error', 'interpreter', 'latex', 'fontsize', FontSize);
set(gca,'fontsize', FontSize-8);

axis tight
% x=input('adjust');

hold off
saveas(gcf, saveNameL1, 'pdf');
close all


%% Frobenius error
for i = 1 : densIters
    for j = 1 : nInstances
        resultsCap(i, j) = frobRelative(Aorig{i}{j}, MaxProduct(Bcap{i}{j}, Ccap{i}{j}));
        resultsNMF(i, j) = frobRelative(Aorig{i}{j}, Bnmf{i}{j}*Cnmf{i}{j});
        resultsSVD(i, j) = frobRelative(Aorig{i}{j}, Bsvd{i}{j}*Csvd{i}{j});
    end
end

errorbar(densities, mean(resultsCap, 2), std(resultsCap, 0, 2), '-xr', ...
    'LineWidth', LW, 'markers', MarkerSize);
hold on
errorbar(densities, mean(resultsNMF, 2), std(resultsNMF, 0, 2), '-xb', ...
    'LineWidth', LW, 'markers', MarkerSize);
errorbar(densities, mean(resultsSVD, 2), std(resultsSVD, 0, 2), '-xk', ...
    'LineWidth', LW, 'markers', MarkerSize);
if addLegend == 1
    h = legend('Capricorn', 'NMF', 'SVD', 'fontsize', FontSize+30);
    set(h,'FontSize', FontSize - mn);
    h.Position(1) = h.Position(1) + xLegendShift;
    h.Position(2) = h.Position(2) + yLegendShift;
    legend boxoff
end

xlabel('Factor density', 'interpreter', 'latex', 'fontsize', FontSize);
ylabel('Frobenius error', 'interpreter', 'latex', 'fontsize', FontSize);
set(gca,'fontsize', FontSize-8);

axis tight
% x=input('adjust');

hold off
saveas(gcf, saveNameFro, 'pdf');
close all
% print('-depsc', '-r300', [saveNameFro '.pdf']);

end


function err_ = frobRelative(A, A_)
err_ = norm(A-A_, 'fro') / norm(A, 'fro');
end




