function plotNoise(minNoise, noiseInc, noiseIters, nInstances, dataName, nmfName, svdName, capName, saveNameL1, saveNameFro)

% Params.
FontSize = 30;
mn = 10;
LW = 1.2;
MarkerSize = 10;

% Find noise range.
noise = nan(noiseIters, 1);
for i = 1 : noiseIters
    noise(i) = minNoise + (i-1)*noiseInc;
end

load(dataName);

load(capName);
load(nmfName);
load(svdName);

resultsCap = nan(noiseIters, nInstances);
resultsNMF = nan(noiseIters, nInstances);
resultsSVD = nan(noiseIters, nInstances);

%% L1 error
for i = 1 : noiseIters
    for j = 1 : nInstances
        resultsCap(i, j) = L1(Aorig{i}{j}, Bcap{i}{j}, Ccap{i}{j}, 0);
        resultsNMF(i, j) = sum(sum(abs(Aorig{i}{j} - Bnmf{i}{j}*Cnmf{i}{j}))) / sum(sum(abs(Aorig{i}{j})));
        resultsSVD(i, j) = sum(sum(abs(Aorig{i}{j} - Bsvd{i}{j}*Csvd{i}{j}))) / sum(sum(abs(Aorig{i}{j})));
    end
end
errorbar(noise, mean(resultsCap, 2), std(resultsCap, 0, 2), '-xr', ...
    'LineWidth', LW, 'markers', MarkerSize);
hold on
errorbar(noise, mean(resultsNMF, 2), std(resultsNMF, 0, 2), '-xb', ...
    'LineWidth', LW, 'markers', MarkerSize);
errorbar(noise, mean(resultsSVD, 2), std(resultsSVD, 0, 2), '-xk', ...
    'LineWidth', LW, 'markers', MarkerSize);

xlabel('Noise', 'interpreter', 'latex', 'fontsize', FontSize);
ylabel('L1 error', 'interpreter', 'latex', 'fontsize', FontSize);
set(gca,'fontsize', FontSize-8);
axis tight

hold off
saveas(gcf, saveNameL1, 'pdf');
close all


%% Frobenius error
for i = 1 : noiseIters
    for j = 1 : nInstances
        resultsCap(i, j) = frobRelative(Aorig{i}{j}, MaxProduct(Bcap{i}{j}, Ccap{i}{j}));
        resultsNMF(i, j) = frobRelative(Aorig{i}{j}, Bnmf{i}{j}*Cnmf{i}{j});
        resultsSVD(i, j) = frobRelative(Aorig{i}{j}, Bsvd{i}{j}*Csvd{i}{j});
    end
end

errorbar(noise, mean(resultsCap, 2), std(resultsCap, 0, 2), '-xr', ...
    'LineWidth', LW, 'markers', MarkerSize);
hold on
errorbar(noise, mean(resultsNMF, 2), std(resultsNMF, 0, 2), '-xb', ...
    'LineWidth', LW, 'markers', MarkerSize);
errorbar(noise, mean(resultsSVD, 2), std(resultsSVD, 0, 2), '-xk', ...
    'LineWidth', LW, 'markers', MarkerSize);

xlabel('Noise', 'interpreter', 'latex', 'fontsize', FontSize);
ylabel('Frobenius error', 'interpreter', 'latex', 'fontsize', FontSize);
set(gca,'fontsize', FontSize-8);

axis tight

hold off
saveas(gcf, saveNameFro, 'pdf');
close all

end


function err_ = frobRelative(A, A_)
err_ = norm(A-A_, 'fro') / norm(A, 'fro');
end




